function pwv, pwv=pwv_in, colden=colden_in, tau225=tau225_in, $
                 tau183=tau183_in, out_colden=out_colden, $
                 out_tau225=out_tau225, out_tau183=out_tau183

;PWV.PRO: calculate Precipitable Water Vapor column from various
;         observables or other parameters, or the inverse.
;
; Calling: out=pwv(
;
;             pwv=...     INPUT PWV in mm
;
;             colden=...  INPUT H2O column density in molecules/m2 
;
;             tau225-...  INPUT opacity (tau) at 225 GHz
;
;             tau183=...  INPUT opacity (tau) at 183 GHz 
;
;             /out_colden OUTPUT H2O column density in molecules/m2 
;
;             /out_tau225 OUTPUT opacity (tau) at 225 GHz 
;
;             /out_tau183 OUTPUT opacity (tau) at 183 GHz
;
;                 )
;
; Output: PWV in mm [DEFAULT]
;
;         H2O column density in molecules/m2 [if /out_colden]
;
;         opacity (tau) at 225 GHz [if /out_tau225]
;
;         opacity (tau) at 183 GHz [if /out_tau183]
;
;
;
; v0.0 20/Dec/2021 A. Boogert: created
;

;cannot give more than 1 input parameter
IF keyword_set(pwv_in)+keyword_set(colden_in)+$
   keyword_set(tau225_in)+keyword_set(tau183_in) GT 1 THEN BEGIN
   print, 'STOP, not more than 1 input option allowed'
   return, 0
ENDIF   

;check output options
IF keyword_set(out_colden) THEN out_colden=1 ELSE out_colden=0
IF keyword_set(out_tau225) THEN out_tau225=1 ELSE out_tau225=0
IF keyword_set(out_tau183) THEN out_tau183=1 ELSE out_tau183=0
;
;...not more than 1 input parameter allowed
outsum=out_colden+out_tau225+out_tau183
IF outsum GT 1 THEN BEGIN
   print, 'STOP, not more than 1 output option allowed'
   return, 0
ENDIF
;
;...default is PWV output
IF outsum EQ 0 THEN out_pwv=1 ELSE out_pwv=0

;conversion factors
;
;...PWV in mm= X * H2O column density [molecules/m2] + Y
;   this assume a liquid water density of 1 gram/cm3 and a molecular
;   weight of water of 18.01528 gram/mole. The constant Y is for
;   consistency with the other formulae
x_colden_pwv=1./3.3428e+25
y_colden_pwv=0.
;
;...PWV in mm= X * Tau_225 + Y
;   Eq. 2 in Dempsey et al. 2013: tau225=0.04*PWV+0.017. PWV is
;   derived from the JCMT 183 GHz water line monitor
;
;   Note that Villanuevo et al. (2018) quote tau225=0.0642*PWV.  The
;   ratio (0.04*PWV+0.017)/(0.0642*PWV) is 1.15 at 0.5 mm, 0.89 at 1.0
;   mmm, 0.80 at 1.5 mm, 0.76 at 2.0 mm, 0.71 at 3.0 mm, and 0.69 at
;   4.0 mm
x_tau225_pwv=1./0.04
y_tau225_pwv=-0.017/0.04
;
;...Relation between tau183 and tau220 (not tau225) from Tamura et
;   al. (2011) is tau(220)=0.186*tau(183)+0.011
;
x_tau183_tau220=0.186
y_tau183_tau220=0.011


IF keyword_set(colden_in) THEN BEGIN
   colden=colden_in
   IF keyword_set(out_colden) THEN BEGIN
      print, 'STOP, output option has to be different from input option'
      return, 0
   ENDIF

   IF keyword_set(out_pwv) THEN BEGIN
      ;colden-->pwv
      out=colden*x_colden_pwv+y_colden_pwv
   ENDIF

   IF keyword_set(out_tau225) THEN BEGIN
      ;colden-->PWV
      out=colden*x_colden_pwv+y_colden_pwv
      ;PWV-->tau225
      out=(out-y_tau225_pwv)/x_tau225_pwv
   ENDIF

   IF keyword_set(out_tau183) THEN BEGIN
      ;colden-->PWV
      out=colden*x_colden_pwv+y_colden_pwv
      ;PWV-->tau225
      out=out*x_tau225_pwv+y_tau225_pwv
      ;tau225-->tau183
      out=(out-y_tau183_tau220)/x_tau183_tau220
   ENDIF
ENDIF

IF keyword_set(pwv_in) THEN BEGIN
   pwv=pwv_in
   IF keyword_set(out_pwv) THEN BEGIN
      print, 'STOP, output option has to be different from input option'
      return, 0
   ENDIF

   IF keyword_set(out_colden) THEN BEGIN
      ;PWV-->colden
      out=(pwv-y_colden_pwv)/x_colden_pwv
   ENDIF

   IF keyword_set(out_tau225) THEN BEGIN
      ;PWV-->tau225
      out=(pwv-y_tau225_pwv)/x_tau225_pwv
   ENDIF

   IF keyword_set(out_tau183) THEN BEGIN
      ;PWV-->tau225
      out=(pwv-y_tau225_pwv)/x_tau225_pwv
      ;tau225-->tau183
      out=(out-y_tau183_tau220)/x_tau183_tau220
   ENDIF
ENDIF

IF keyword_set(tau225_in) THEN BEGIN
   tau225=tau225_in
   IF keyword_set(out_tau225) THEN BEGIN
      print, 'STOP, output option has to be different from input option'
      return, 0
   ENDIF

   IF keyword_set(out_pwv) THEN BEGIN
      ;tau225-->PWV
      out=x_tau225_pwv*tau225+y_tau225_pwv
   ENDIF

   IF keyword_set(out_colden) THEN BEGIN
      ;tau225-->PWV
      out=x_tau225_pwv*tau225+y_tau225_pwv
      ;PWV-->colden
      out=(out-y_colden_pwv)/x_colden_pwv
   ENDIF

   IF keyword_set(out_tau183) THEN BEGIN
      ;tau225-->tau183
      out=(tau225-y_tau183_tau220)/x_tau183_tau220
   ENDIF
ENDIF

IF keyword_set(tau183_in) THEN BEGIN
   tau183=tau183_in
   IF keyword_set(out_tau183) THEN BEGIN
      print, 'STOP, output option has to be different from input option'
      return, 0
   ENDIF

   IF keyword_set(out_tau225) THEN BEGIN
      ;tau183-->tau225
      out=x_tau183_tau220*tau183+y_tau183_tau220
   ENDIF

   IF keyword_set(out_pwv) THEN BEGIN
      ;tau183-->tau225
      out=x_tau183_tau220*tau183+y_tau183_tau220
      ;tau225-->PWV
      out=x_tau225_pwv*out+y_tau225_pwv
   ENDIF

   IF keyword_set(out_colden) THEN BEGIN
      ;tau183-->tau225
      out=x_tau183_tau220*tau183+y_tau183_tau220
      ;tau225-->PWV
      out=x_tau225_pwv*out+y_tau225_pwv
      ;PWV-->colden
      out=(out-y_colden_pwv)/x_colden_pwv
   ENDIF

ENDIF


return, out

end
